#!/usr/bin/env python
# vim: et sta sts ai ts=4 sw=4:
import os
import subprocess
import sys
import re

from charmhelpers.core import hookenv
from charmhelpers.core.hookenv import log
from charmhelpers.contrib.cloudfoundry import contexts
from charmhelpers.contrib.cloudfoundry import services
from config import *


def default_domain():
    public_address = hookenv.unit_get('public-address')
    if not re.match('^(\d{1,3}\.){3}\d{1,3}$', public_address):
        result = subprocess.check_output([
            'dig', '+short', '@8.8.8.8', public_address])
        public_address = result.strip()
    domain = "%s.xip.io" % public_address
    return domain


def get_domain():
    config = hookenv.config()
    domain = config.get('domain')
    if not domain:
        domain = default_domain()
    return domain


hooks = hookenv.Hooks()
fileproperties = {'owner': 'vcap', 'mode': '644'}

services.register([
    {
        'service': 'gorouter',
        'templates': [{
            'source': 'gorouter.yml',
            'target': ROUTER_CONFIG_FILE,
            'file_properties': fileproperties,
            'contexts': [
                contexts.StaticContext({'domain': default_domain()}),
                contexts.ConfigContext(),
                contexts.NatsContext()
            ]
        }]
    }
], os.path.join(hookenv.charm_dir(), 'templates'))


@hooks.hook()
def start():
    hookenv.open_port(80)
    hookenv.open_port(443)


@hooks.hook("config-changed")
def config_changed():
    services.reconfigure_services()
    domain = get_domain()
    for rid in hookenv.relation_ids('router'):
        hookenv.relation_set(rid, {'domain': domain})


@hooks.hook()
def stop():
    services.stop_services()
    hookenv.close_port(80)
    hookenv.close_port(443)


@hooks.hook('nats-relation-changed')
def nats_relation_changed():
    services.reconfigure_services()


@hooks.hook('router-relation-joined')
def router_relation_joined():
    domain = get_domain()
    hookenv.relation_set(None, {'domain': domain})


if __name__ == '__main__':
    hook_name = os.path.basename(sys.argv[0])
    log("Running {} hook".format(hook_name))
    if hookenv.relation_id():
        log("Relation {} with {}".format(
            hookenv.relation_id(), hookenv.remote_unit()))
    hooks.execute(sys.argv)
